"""Bileto Actions

Define the actions available to perform on a ticket.
"""


class Actions:
    """Define what actions can be taken on a ticket."""
    statuses = dict(
        status='Updating status.',
        build='Preparing packages.',
        diff='Generating diffs.',
        publish='Publishing packages.',
        finalize='Merging to trunk.',
        abandon='Abandoning ticket.',
    )

    @staticmethod
    def status(manager):
        """Update Package States"""
        manager.status()

    @staticmethod
    def build(manager):
        """Build Packages"""
        manager.validate().collect().build().upload().expose().diff()

    @staticmethod
    def diff(manager):
        """Regenerate Diffs"""
        manager.diff()

    @staticmethod
    def publish(manager):
        """Copy Packages to Destination Archive"""
        manager.checkstatus().validate().checkupload().ackaging()
        manager.checkdest().unapproved().unbuilt().publish()

    @staticmethod
    def finalize(manager):
        """Merge to Trunk"""
        manager.merge().push().nuke()

    @staticmethod
    def abandon(manager):
        """Delete PPA"""
        manager.abandon().nuke()
