#!/usr/bin/python3
"""Bileto App

Initiate the Flask app, and connect to the database.
"""

from datetime import timedelta

from flask import Flask, Request
from flask.ext.openid import OpenID
from flask.ext.sqlalchemy import SQLAlchemy

from bileto.json import BiletoJSONEncoder
from bileto.settings import Config


DEVEL = Config.ppa_team == Config.launchpad_user
STAGING = 'staging' in Config.ppa_team


class ProxiedRequest(Request):
    """Teach Flask how to behave behind an SSL reverseproxy."""
    def __init__(self, environ, populate_request=True, shallow=False):
        super().__init__(environ, populate_request, shallow)
        for attr in ('url', 'host_url', 'base_url', 'url_root'):
            https = getattr(self, attr, '').replace('http://', 'https://')
            setattr(self, attr, https)


app = Flask(__name__)
DEFAULT_REQUEST = app.request_class
app.request_class = ProxiedRequest
app.json_encoder = BiletoJSONEncoder
app.config['JSONIFY_PRETTYPRINT_REGULAR'] = DEVEL or STAGING

db_uri = Config.postgresql_info or 'sqlite:////tmp/bileto.db'
app.config['SQLALCHEMY_DATABASE_URI'] = db_uri

app.config['SEND_FILE_MAX_AGE_DEFAULT'] = timedelta(minutes=30).seconds

app.secret_key = Config.session_key

db = SQLAlchemy(app)

oid = OpenID(app, Config.OPENID_PATH, safe_roots=[])


@app.teardown_appcontext
def shutdown_session(exception=None):
    """Automatically rollback db transaction when request context ends."""
    db.session.remove()
