"""Bileto Authentication

Define some basic authentication behavior.
"""

from flask import abort, request, session

from bileto.settings import Config


ADMIN_TEAMS = {
    'canonical-platform-qa',
    'ci-train-ppa-service',
    'ubuntu-core-dev',
}
USER_TEAMS = ADMIN_TEAMS | {'ci-train-users'}


def check_api_token():
    """Confirm if the appropriate API token has been submitted."""
    token = Config.api_token
    return token and token == request.args.get('token')


def has_team(teams):
    """Return the set of teams that our user is actually in, if any."""
    return teams & set(session.get('teams') or set())


def require_team(teams):
    """Abort if logged-in user is not in at least one of the given teams."""
    if not has_team(teams or USER_TEAMS):
        abort(401)


def authenticate():
    """Prevent API from being accessed without correct authentication."""
    if not (has_team(USER_TEAMS) or check_api_token()):
        abort(401)
