"""Bileto Disconnect Handling.

This will attempt to reconnect if the connection to postgres is lost.
"""

import logging

from sqlalchemy import exc
from sqlalchemy import event
from sqlalchemy.pool import Pool


@event.listens_for(Pool, 'checkout')
def ping_connection(dbapi_connection, connection_record, connection_proxy):
    """Ping db and reconnect if the connection has died."""
    cursor = dbapi_connection.cursor()
    try:
        cursor.execute('SELECT 1')
    except Exception as err:  # pragma: no cover
        logging.error(str(err))
        # Pool will try reconnecting three times before raising.
        raise exc.DisconnectionError()
    cursor.close()
