"""Bileto Login

Define the necessary API endpoints to make OpenID function correctly.
"""

from flask import request, session, redirect

from bileto.app import app, oid
from bileto.lplib import lp


@app.route('/login', methods=['GET', 'POST'])
@oid.loginhandler
def login():
    """Initiate OpenID login."""
    if 'nickname' in session:
        return redirect(oid.get_next_url())
    if 'next' in request.form:
        return oid.try_login(
            'https://login.ubuntu.com/',
            ask_for=['nickname'],
            ask_for_optional=['fullname', 'email'])
    return redirect('/')


@oid.after_login
def identify(resp):
    """Complete OpenID login."""
    nick = resp.nickname
    session.update(
        ircnick=lp.get_irc_nick(nick),
        teams=lp.get_teams(nick),
        identity_url=resp.identity_url,
        fullname=resp.fullname,
        nickname=nick,
        email=resp.email,
    )
    return redirect(oid.get_next_url())


@app.route('/logout')
def logout():
    """Clear user session, logging them out."""
    session.clear()
    return redirect(oid.get_next_url())
