"""Bileto Settings

Define some global settings used by various parts of Bileto.
"""

from os.path import abspath, expanduser, dirname, join
from contextlib import suppress
from functools import lru_cache

UNDERSCORE = '_'
TILDE = '~'
EMPTY = ''
DOT = '.'


class Config:
    """Read configuration values from disk."""
    OPENID_PATH = '/tmp/bileto/openid'
    HOME = expanduser(TILDE)
    ROOT = dirname(dirname(abspath(__file__)))

    LEVELS = ('info', 'debug')
    GLES = '-gles'

    dirs = ('/etc/bileto', HOME)
    prefixes = (EMPTY, DOT, 'bileto.')

    @lru_cache()
    def __getitem__(self, key):
        """Read configuration value from disk."""
        for prefix in self.prefixes:
            for root in self.dirs:
                with suppress(FileNotFoundError):
                    path = join(root, prefix + key)
                    with open(path, encoding='utf-8') as data:
                        return data.read().strip()
        return ''

    @lru_cache()
    def __getattr__(self, key):
        """Read configuration value from disk."""
        return self[key] or self[key.replace(UNDERSCORE, DOT)]

Config = Config()  # Single global instance
