var EXCUSES = 'http://people.canonical.com/~ubuntu-archive/proposed-migration/';
var PPA_PREFIX = 'https://launchpad.net/~';
var SOURCE_PREFIX = 'https://launchpad.net/ubuntu/+source/';
var TEAM_PREFIX = 'https://launchpad.net/~';

var visibilitychange = null;
var hidden = null;
['h', 'mozH', 'webkitH', 'msH', 'oH'].some(function(prefix) {
    hidden = prefix + 'idden';
    if (hidden in document) {
        visibilitychange = prefix.slice(0, -1) + 'visibilitychange';
        return true; // Found it, break loop!
    }
});

var app = angular.module('Bileto', ['ngSanitize']);

function a(href, text) {
    if (!href) return '';
    return ' <a target="_blank" href="' + href + '">' + text + '</a>';
}

function shorter(url) {
    var parts = url.split('/').splice(3);
    if (parts[3] == '+merge') {
        parts = parts.splice(1, 2);
    }
    if ((parts[1] || '').lastIndexOf('AUTH_', 0) === 0) {
        parts = parts.splice(3);
    }
    var short = parts.join(' ').replace(/_/g, ' ');
    return a(url, short);
}

function linkify(text) {
    text = text || '';
    return text.replace(/\n/g, '<br />')
               .replace(/(#|bug)\s?(\d+)/g, ' http://pad.lv/$2')
               .replace(/\/static\/[-A-Z0-9+&@#/%?=~_|!:,.;]*[A-Z0-9+&@#/%=~_|]/gi, shorter)
               .replace(/https?:\/\/[-A-Z0-9+&@#/%?=~_|!:,.;]*[A-Z0-9+&@#/%=~_|]/gi, shorter);
}

function link_each_word(prefix, words) {
    words = (words || '').split(/[\s,]+/);
    text = '';
    for (var i=0; i < words.length; i++) {
        text += a(prefix + words[i], words[i]);
    }
    return text;
}

function display_siloname(ppa_team, text) {
    text = (text || '');
    return a(PPA_PREFIX + ppa_team + '/+archive/' + text + '/+packages', text)
}

function link_excuses(match, packagelist) {
    var text = '';
    var packages = packagelist.split(', ');
    for (var i=0; i < packages.length; i++) {
        var parts = packages[i].split('/');
        var name = parts[0];
        var series = parts[1];
        if (i>0) text += ', ';
        text += a(EXCUSES + series + '/update_excuses.html#' + name, packages[i]);
    }
    return 'Proposed pocket (' + text + ')';
}

function display_status(text) {
    text = linkify((text || ''));
    text = text.replace(/Proposed pocket \(([^)]+)\)/gi, link_excuses);
    return text.replace(/\. /g, '.<br/>')
}

function display_landers(text) {
    return link_each_word('/#/user/', (text || ''));
}

function display_sources(text) {
    return link_each_word(SOURCE_PREFIX, (text || ''));
}

function display_sru_warning(req) {
    // Vivid hard-coded to overlay PPA now.
    if (req.series == "vivid") return false;
    // Dual silos are never SRUs.
    if (req.series.indexOf('+') >= 0) return false;
    // Display SRU warning for non-yakkety silos with no PPA target
    return req.series != "yakkety" && !req.dest;
}

function display_dest(text) {
    if (!text) return 'Ubuntu Archive';
    var parts = text.split(/\//);
    return a(TEAM_PREFIX + parts[0] + '/+archive/' + parts[1] + '/' + parts[2], parts[2]);
}

function title(text) {
    return (text || '\n').split('\n')[0];
}

app.controller('appController', ['$scope', '$location', '$http', '$anchorScroll', function($scope, $location, $http, $anchorScroll) {
    $scope.long_fields = ['description', 'test_plan', 'merge_proposals', 'download_links'];
    $scope.dropdowns = ['series'];

    $scope.linkify = linkify;
    $scope.display_siloname = display_siloname;
    $scope.display_status = display_status;
    $scope.display_landers = display_landers;
    $scope.display_sources = display_sources;
    $scope.display_dest = display_dest;
    $scope.display_sru_warning = display_sru_warning;
    $scope.title = title;

    $scope.full_log = false;
    $scope.editing = null;
    $scope.busy = false;
    $scope.expand = false;
    $scope.error = null;

    $scope.meta = {};
    $scope.data = {'requests': {}, 'assigned': 0};
    $scope.new_comment = {'text': ''};

    function set_error(status, header, text) {
        $scope.busy = false;
        $scope.data.next = null;
        $scope.data.prev = null;
        $scope.data.requests = [];
        $scope.error = {
            'status': status || 0,
            'header': header,
            'message': text,
        };
    }

    function generic_error(data, status, headers, config) {
        var unresponsive = 'Backend not responding. Most likely Bileto is in the middle of a new rollout, please try again in a couple minutes.';
        var message = {
            0: 'I think this means your request was interrupted by you. Stop clicking so much! Alert robru if problem persists.',
            400: 'You submitted a malformed request. Were all the fields on the form filled out before you submitted? Did you manually mangle the URL in a bad way?',
            401: 'You are not authorized to access this page. Are you logged in? Are you a member of the correct teams?',
            404: 'This page could not be found. Please check the URL and try again.',
            405: 'Method not allowed. It seems you POSTed when you meant to GET... or maybe the other way around.',
            500: 'Something has gone horribly wrong. Ask robru to investigate.',
            502: unresponsive,
            503: unresponsive,
        }[status] || 'This error was not anticipated by robru. You should definitely let him know immediately.';
        set_error(status, status + ' ERROR', message);
    }

    $http.get('/v1/metadata').success(function(data, status, headers, config) {
        $scope.meta = data;
    }).error(generic_error);

    $scope.toggle_log = function() {
        $scope.full_log = !$scope.full_log;
        $location.hash($scope.full_log ? 'audit_log' : '')
    }

    $scope.request = function(value) {
        $location.path('/ticket/' + value);
        $location.search('');
        $scope.refresh();
    }

    $scope.siloname = function(value) {
        if (value.match(/^\d+$/)) {
            value = value.replace(/^0+/, '');
            while (value.length <3) {
                value = '0' + value;
            }
        }
        $location.path('/silo/' + value);
        $location.search('');
        $scope.refresh();
    }

    $scope.search = function(value, active) {
        $location.path('/tickets');
        $location.search('');
        if (active) {
            $location.search('active_only');
        }
        $location.search('search', value);
        $scope.refresh();
    }

    $scope.next = function() {
        $location.search($scope.data.next);
        $anchorScroll();
    }

    $scope.prev = function() {
        $location.search($scope.data.prev);
    }

    $scope.refresh = function() {
        if (document[hidden]) return;
        if ($scope.editing) return;
        $scope.full_log = $location.hash() == 'audit_log';
        $scope.here = $location.url();
        var params = $location.search();
        var path = $location.path();
        var url = '/v1' + path;
        if (url.length < 6) {
            url = '/v1/tickets';
        }
        if (url == '/v1/tickets') {
            params.limit = 50;
        }
        $scope.error = null;
        $http.get(url, {'params': params}).success(function(data) {
            if ($scope.data.instance && $scope.data.instance != data.instance) {
                // There's been a new train rollout, reload page to get latest HTML.
                location.reload(true);
            }
            $scope.data = data;
            if (!data.requests.length) {
                set_error(0, 'No Results', 'This search returned no results. Please try again with less-specific search terms.');
            }
            $scope.expand = data.requests.length == 1;
            if (url == '/v1/create') {
                $scope.editing = true;
            }
        }).error(generic_error);
    }

    $scope.submit = function(form, req) {
        var dirty_bits = {request_id: req.request_id};
        for (var field in form) {
            if (form[field] && form[field].$dirty || !req.request_id) {
                dirty_bits[field] = req[field];
            }
        }
        $scope.busy = true;
        $http.post('/v1/tickets', dirty_bits).success(function(data, status, headers, config) {
            $scope.editing = null;
            $location.path('/ticket/' + data.request_id);
            $location.search('');
            $scope.busy = false;
            $scope.refresh();
        }).error(generic_error);
    }

    $scope.cancel = function() {
        $scope.editing = null;
        $scope.refresh();
    }

    $scope.edit = function(req) {
        if ($scope.editing) {
            alert('Please save or cancel existing edit.')
        } else {
            $scope.editing = req;
        }
    }

    $scope.post_comment = function(req_id, text) {
        $scope.busy = true;
        $http.post('/v1/comment', {
            request_id: req_id,
            text: text,
        }).success(function(data, status, headers, config) {
            $scope.new_comment.text = '';
            $scope.busy = false;
            $scope.refresh();
        }).error(generic_error);
    }

    $scope.$watch(function() { return $location.url(); }, function() {
        $scope.editing = false;
        $scope.refresh();
    });

    setInterval($scope.refresh, 5 * 60 * 1000);
    if (visibilitychange) {
        document.addEventListener(visibilitychange, $scope.refresh);
    }

    $scope.easter_egg = function(number) {
        var str = number.toString();
        var len = str.length - 1;
        var i = str.length;
        while (i--) {
            if (str.charAt(i) !== str.charAt(len - i)) {
                return '';
            }
        }
        return '❤️';
    }
}]);
