"""Define all version-handling logic."""

import re

from subprocess import Popen
from datetime import datetime


SERIES_DATE = re.compile(r'^(.*\+)?(\d\d\.\d\d)\.(\d{8}(.\d+)?)')
TILDE = '~'
COLON = ':'
EMPTY = ''
DASH = '-'
PLUS = '+'


def comparison(one, comp, two):
    """Compare version a against version b."""
    # I think this is fast enough that asyncio is not necessary...
    process = Popen(('dpkg', '--compare-versions', str(one), comp, str(two)))
    process.communicate()
    return process.returncode == 0


def extreme(edge, versions):
    """Find the max or min of a given list of version strings."""
    return str(edge([V(v) for v in versions]))


def plus_one(version):
    """Increment the minor component of a version number."""
    base, div, point = version.partition('.')
    point = str(int(point) + 1 if point else 1)
    return '{base}.{point}'.format(**locals())


class V(object):
    """Define version number comparison operations."""

    def __init__(self, version):
        self.v = version

    def __str__(self):
        return self.v or EMPTY

    def create_new(self, series):
        """Create a new version guaranteed to be higher than the given one."""
        ubu = '-0ubuntu1' if 'ubuntu' in self.v else EMPTY
        if PLUS not in self.v and 'ubuntu' not in self.v:
            ubu = '-0ubuntu1'
        base = self.v.partition(DASH)[0]
        base = base.partition('is.')[0]
        base, _, date = base.partition(PLUS)
        base = base.partition('ubuntu')[0]
        rev, old_series = EMPTY, EMPTY
        match = SERIES_DATE.match(date)
        if match:
            rev = match.group(1) or EMPTY
            old_series = match.group(2) or EMPTY
            date = match.group(3) or '0'
        day = V.max(datetime.utcnow().strftime('%Y%m%d'), date)
        if day == date and series == old_series:
            day = plus_one(day)
        return '{base}+{rev}{series}.{day}{ubu}'.format(**locals())

    @staticmethod
    def max(*versions):
        """Return the highest version number."""
        return extreme(max, versions)

    def __gt__(self, other):
        return comparison(self, 'gt', other)
