#!/usr/bin/env python3

"""Run Bileto

Create db tables on first run, otherwise init bileto app.
"""

from datetime import datetime

from sqlalchemy.exc import OperationalError, ProgrammingError

from bileto.models import Request
from bileto.app import DEFAULT_REQUEST, app, db
from bileto import login, v1, disconnect, streams

# These register themselves with flask at import time
# This is just here to appease pyflakes.
MODULES = [login, v1, disconnect, streams]


try:
    Request.query.get(1)
except (OperationalError, ProgrammingError):
    # Create db tables on first run.
    db.create_all()


print('{} Connected to {}.'.format(
    datetime.utcnow(),
    app.config['SQLALCHEMY_DATABASE_URI'].partition(':')[0]))

if __name__ == '__main__':
    app.request_class = DEFAULT_REQUEST
    app.run(debug=True)
