"""Bileto Actions Test

Test the defined ticket actions.
"""

from unittest.mock import Mock, call

from tests.tests import WorkerTestCase

from bileto.actions import Actions


class MainTestCase(WorkerTestCase):
    """Test Main Subprocess Script."""

    def test_actions_status(self):
        """Ensure status calls correct methods."""
        manager = Mock()
        Actions.status(manager)
        self.assertSequenceEqual(manager.mock_calls, [
            call.status(),
        ])

    def test_actions_diff(self):
        """Ensure diff calls correct methods."""
        manager = Mock()
        Actions.diff(manager)
        self.assertSequenceEqual(manager.mock_calls, [
            call.diff(),
        ])

    def test_actions_publish(self):
        """Ensure publish calls correct methods."""
        manager = Mock()
        Actions.publish(manager)
        self.assertSequenceEqual(manager.mock_calls, [
            call.checkstatus(),
            call.checkstatus().validate(),
            call.checkstatus().validate().checkupload(),
            call.checkstatus().validate().checkupload().ackaging(),
            call.checkdest(),
            call.checkdest().unapproved(),
            call.checkdest().unapproved().unbuilt(),
            call.checkdest().unapproved().unbuilt().publish(),
        ])

    def test_actions_finalize(self):
        """Ensure finalize calls correct methods."""
        manager = Mock()
        Actions.finalize(manager)
        self.assertSequenceEqual(manager.mock_calls, [
            call.merge(),
            call.merge().push(),
            call.merge().push().nuke(),
        ])

    def test_actions_abandon(self):
        """Ensure abandon calls correct methods."""
        manager = Mock()
        Actions.abandon(manager)
        self.assertSequenceEqual(manager.mock_calls, [
            call.abandon(),
            call.abandon().nuke(),
        ])
