"""Bileto App Tests

Test for various functions found in app.py.
"""

from glob import glob
from subprocess import Popen

from bileto import v1
from bileto.settings import Config

from tests.tests import BiletoTestCase

MODULES = [v1]


class AppTestCase(BiletoTestCase):
    """Test a few random app functions."""

    def test_static_app(self):
        """Ensure front page gives us Angular app."""
        ret = self.app.get('/')
        self.assertIn(b'ng-app="Bileto"', ret.data)

    def test_read_from_file(self):
        """Ensure that we can read files from disk correctly."""
        Config.dirs = ('/etc/python3/',)
        self.assertIn('standard', Config['debian_config'])

    def test_imports(self):
        """Ensure long-running code doesn't import any short-lived code."""
        proc = Popen(['grep', 'bileto.worker'] + glob('bileto/*.py'))
        proc.communicate()
        self.assertEqual(
            proc.returncode, 1, 'bileto.worker imported outside bileto.worker')
