"""Bileto GLES Test

Test that GLES behaves correctly.
"""

from unittest.mock import call, patch
from subprocess import PIPE
from os.path import join

from tests.tests import WorkerTestCase, fake_subproc

from bileto.settings import Config
from bileto.worker.vcs import VCSError
from bileto.worker.gles import GLES


FAKE_SUBPROC = fake_subproc(0, [(b'', b'')])
ERROR = fake_subproc(1, [(b'error message', b'logged')])


class GLESTestCase(WorkerTestCase):
    """Test GLES Packages."""

    @patch('bileto.worker.vcs.create_subprocess_exec', FAKE_SUBPROC)
    def test_gles_collect(self):
        """Invoke gles collection correctly."""
        gles = GLES('qt-gles', self.distro, self.series, self.dest, self.ppa)
        self.run_coro(gles.collect())
        env = FAKE_SUBPROC.mock_calls[0][2]['env']
        self.assertEqual(env['SOURCE'], 'qt-gles')
        self.assertEqual(env['SERIES'], 'xenial')
        self.assertEqual(env['ACTION'], 'get_nongles')
        self.assertSequenceEqual(FAKE_SUBPROC.mock_calls, [
            call(join(Config.ROOT, 'scripts', 'vcs.sh'),
                 env=env, stdout=PIPE, stderr=PIPE),
        ])

    @patch('bileto.worker.vcs.create_subprocess_exec', ERROR)
    def test_gles_collect_error(self):
        """Invoke gles collection correctly."""
        gles = GLES('qt-gles', self.distro, self.series, self.dest, self.ppa)
        with self.assertRaisesRegex(VCSError, 'error message'):
            self.run_coro(gles.collect())
        env = ERROR.mock_calls[0][2]['env']
        self.assertEqual(env['SOURCE'], 'qt-gles')
        self.assertEqual(env['SERIES'], 'xenial')
        self.assertEqual(env['ACTION'], 'get_nongles')
        self.assertSequenceEqual(ERROR.mock_calls, [
            call(join(Config.ROOT, 'scripts', 'vcs.sh'),
                 env=env, stdout=PIPE, stderr=PIPE),
        ])
