"""Bileto Secondary Test

Test that Secondary builds behave correctly.
"""

from os.path import join
from subprocess import PIPE
from unittest.mock import call, patch

from tests.tests import WorkerTestCase, fake_subproc

from bileto.settings import Config
from bileto.worker.secondary import Secondary


FAKE_SUBPROC = fake_subproc(0, [(b'', b'')])


class SecondaryTestCase(WorkerTestCase):
    """Test Secondary Packages."""

    @patch('bileto.worker.vcs.create_subprocess_exec', FAKE_SUBPROC)
    def test_secondary_collect(self):
        """Invoke gles collection correctly."""
        secondary = Secondary(
            'ofono', self.distro, self.series, self.dest, self.ppa)
        self.run_coro(secondary.collect())
        env = FAKE_SUBPROC.mock_calls[0][2]['env']
        self.assertEqual(env['SOURCE'], 'ofono')
        self.assertEqual(env['SERIES'], 'xenial')
        self.assertEqual(env['ACTION'], 'get_secondary')
        self.assertSequenceEqual(FAKE_SUBPROC.mock_calls, [
            call(join(Config.ROOT, 'scripts', 'vcs.sh'),
                 env=env, stdout=PIPE, stderr=PIPE),
        ])
